(function() {
  var Connection, DBPM, Util;

  Connection = require("tedious").Connection;

  Util = require("./Util.js");

  DBPM = (function() {
    function DBPM() {}

    DBPM.ready = false;

    DBPM.initCount = -1;

    DBPM.configFile = "sconfig";

    DBPM.allocateControl = true;

    DBPM.pool = {};

    DBPM.init = function(callback) {
      var error, server;
      try {
        DBPM.config = require("" + (process.cwd()) + "/" + this.configFile + ".json");
      } catch (_error) {
        error = _error;
        throw new Error("" + this.configFile + ".json not found on root folder");
      }
      for (server in DBPM.config.serverList) {
        DBPM.config.serverList[server].options.appName = "Sapphire";
        DBPM.pool[server] = {};
        DBPM.pool[server].connections = [];
        DBPM.pool[server].queue = [];
      }
      if (callback != null) {
        return DBPM.initializePool(callback);
      }
    };

    DBPM.hasServerAvailable = function(server) {
      if (DBPM.config.serverList[server]) {
        return true;
      } else {
        return false;
      }
    };

    DBPM.initializePool = function(callback) {
      var i, server;
      for (server in DBPM.config.serverList) {
        i = 0;
        while (i < DBPM.config.maxConnections) {
          DBPM.connect(server);
          DBPM.initCount++;
          i++;
        }
      }
      DBPM.initCount = 0;
      return callback.apply();
    };

    DBPM.connect = function(server, serverDownCallback, retries) {
      var connection;
      connection = null;
      if (DBPM.hasServerAvailable(server)) {
        connection = new Connection(DBPM.config.serverList[server]);
        connection.poolServer = server;
        connection.connectionRetries = retries || 0;
        connection.creationTime = Date.now();
        connection.once("connect", function(err) {
          if (err != null) {
            return DBPM.retry(connection, serverDownCallback);
          } else {
            DBPM.allocate(connection);
            return connection.once("end", function() {
              return DBPM.refresh(connection);
            });
          }
        });
      } else {
        throw new Error("" + server + " is not on list");
      }
      return connection;
    };

    DBPM.allocate = function(connection) {
      var id;
      id = DBPM.pool[connection.poolServer].connections.length;
      if (DBPM.allocateControl && DBPM.config.maxConnections === id) {
        DBPM.allocateControl = false;
        return DBPM.processQueue(connection);
      } else {
        connection.poolId = id;
        return DBPM.pool[connection.poolServer].connections.push(connection);
      }
    };

    DBPM.refresh = function(connection) {
      DBPM.pool[connection.poolServer].connections.splice(connection.poolId, 1);
      return DBPM.connect(connection.poolServer);
    };

    DBPM.retry = function(connection, errorCallback) {
      var i;
      i = connection;
      if (i.connectionRetries <= 3) {
        i.connectionRetries++;
        setTimeout(function() {
          return DBPM.connect(i.poolServer, errorCallback, i.connectionRetries);
        }, 1000);
      } else {
        if (Util.typeOf(errorCallback) === "function") {
          errorCallback.apply(this, ["" + i.poolServer + " server unavailable"]);
        }
      }
      return i;
    };

    DBPM.getConnection = function(server) {
      var firstConnection;
      return firstConnection = DBPM.pool[server].connections[0];
    };

    DBPM.addToQueue = function(server, request) {
      return DBPM.pool[server].queue.push(request);
    };

    DBPM.processQueue = function(connection) {
      var nextRequest, server;
      server = connection.poolServer;
      nextRequest = DBPM.pool[server].queue[0];
      if (nextRequest !== void 0) {
        DBPM.pool[server].queue.splice(0, 1);
        nextRequest.once("requestCompleted", function() {
          return DBPM.processQueue(connection);
        });
        return connection.callProcedure(nextRequest);
      } else {
        return DBPM.allocate(connection);
      }
    };

    DBPM.execute = function(server, request) {
      var connection;
      connection = DBPM.getConnection(server);
      if (connection === void 0 || !DBPM.pool) {
        DBPM.addToQueue(server, request);
      } else {
        DBPM.pool[connection.poolServer].connections.splice(this.poolId, 1);
        request.once("requestCompleted", function() {
          return DBPM.processQueue(connection);
        });
        connection.callProcedure(request);
      }
      return connection;
    };

    return DBPM;

  })();

  DBPM.init(function() {
    return DBPM.ready = true;
  });

  module.exports = DBPM;

}).call(this);
