(function() {
  var Util, Validation;

  Util = require("./Util.js");

  Validation = (function() {
    function Validation() {}

    Validation.TYPES = require("../limits_data/limits.json");

    Validation.isSizeOK = function(type, value, minValue, maxValue) {
      var item, _i, _len, _ref;
      if (minValue != null) {
        if (minValue.getTime) {
          minValue = minValue.getTime();
        }
      }
      if (maxValue != null) {
        if (maxValue.getTime) {
          maxValue = maxValue.getTime();
        }
      }
      _ref = Validation.TYPES;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        item = _ref[_i];
        if (type === item.name) {
          if (!minValue || minValue < item.min) {
            minValue = item.min;
          }
          if (!maxValue || maxValue > item.max) {
            maxValue = item.max;
          }
          return Validation.validateSize(value, minValue, maxValue);
        }
      }
      return true;
    };

    Validation.validateSize = function(value, minValue, maxValue) {
      if (Util.typeOf(value) === "string") {
        value = value.length;
      }
      if ((maxValue >= value && value >= minValue)) {
        return true;
      } else {
        return false;
      }
    };

    Validation.isTypeOK = function(type, value) {
      var correctTypes, item, _i, _j, _k, _l, _len, _len1, _len2, _len3;
      if (Util.typeOf(value) === "string") {
        correctTypes = ["varchar", "char", "text", "ntext", "nvarchar", "nchar", "uniqueidentifier"];
        for (_i = 0, _len = correctTypes.length; _i < _len; _i++) {
          item = correctTypes[_i];
          if (item === type) {
            return true;
          }
        }
      }
      if (Util.typeOf(value) === "number") {
        correctTypes = ["int", "tinyint", "smallint", "bigint", "float", "money", "numeric"];
        for (_j = 0, _len1 = correctTypes.length; _j < _len1; _j++) {
          item = correctTypes[_j];
          if (item === type) {
            return true;
          }
        }
      }
      if (Util.typeOf(value) === "boolean") {
        correctTypes = ["bit"];
        for (_k = 0, _len2 = correctTypes.length; _k < _len2; _k++) {
          item = correctTypes[_k];
          if (item === type) {
            return true;
          }
        }
      }
      if (Util.typeOf(value) === "date") {
        correctTypes = ["date", "datetime", "smalldatetime", "time"];
        for (_l = 0, _len3 = correctTypes.length; _l < _len3; _l++) {
          item = correctTypes[_l];
          if (item === type) {
            return true;
          }
        }
      }
      return false;
    };

    Validation.isOnlyNumbers = function(value) {
      return /^[0-9]+$/.test(value);
    };

    Validation.isOnlyLettersAndSymbols = function(value) {
      return /^[^0-9]+$/.test(value);
    };

    Validation.isOnlyLetters = function(value) {
      return /^[a-z\sA-Z]+$/.test(value);
    };

    return Validation;

  })();

  module.exports = Validation;

}).call(this);
