(function() {
  var Connection, DBPM, EventEmitter, Request, Sanitization, Sapphire, Util, Validation, dataTypes,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Connection = require("tedious").Connection;

  Request = require("tedious").Request;

  EventEmitter = require("events").EventEmitter;

  dataTypes = require("tedious").TYPES;

  DBPM = require("./DBPM.js");

  Util = require("./Util.js");

  Sanitization = require("./Sanitization.js");

  Validation = require("./Validation.js");

  Sapphire = (function(_super) {
    __extends(Sapphire, _super);

    Sapphire.parametersTypes = {
      varchar: dataTypes.VarChar,
      text: dataTypes.Text,
      char: dataTypes.VarChar,
      nvarchar: dataTypes.NVarChar,
      ntext: dataTypes.NVarChar,
      nchar: dataTypes.NVarChar,
      bigint: dataTypes.VarChar,
      int: dataTypes.Int,
      smallint: dataTypes.SmallInt,
      tinyint: dataTypes.TinyInt,
      bit: dataTypes.Bit,
      datetime: dataTypes.DateTime,
      smalldatetime: dataTypes.SmallDateTime,
      date: dataTypes.DateTime,
      time: dataTypes.DateTime,
      float: dataTypes.Float,
      numeric: dataTypes.Float,
      money: dataTypes.Float,
      uniqueidentifier: dataTypes.VarChar
    };

    Sapphire.prototype.pq = null;

    function Sapphire(server, sp, parameters) {
      this.prepare(server || null, sp || null, parameters || null);
    }

    Sapphire.prototype.hasDataType = function(type) {
      var item, _i, _len, _ref;
      _ref = Object.keys(Sapphire.parametersTypes);
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        item = _ref[_i];
        if (type === item) {
          return true;
        }
      }
      return false;
    };

    Sapphire.prototype.prepare = function(server, sp, parameters) {
      return this.pq = {
        server: server,
        sp: sp,
        parameters: parameters
      };
    };

    Sapphire.prototype.query = function(server, sp, parameters) {
      var parameter, request, resultArray, rowIndex, sanitizedParameters, _i, _len,
        _this = this;
      resultArray = [];
      rowIndex = 0;
      if (DBPM.hasServerAvailable(server)) {
        if (parameters != null) {
          sanitizedParameters = this.sanitize(parameters);
        } else {
          sanitizedParameters = [];
        }
        if (this.isValid(sanitizedParameters)) {
          request = new Request(sp, function(err) {
            if (err != null) {
              return _this.emit("databaseError", err, server, sp);
            } else {
              return _this.emit("results", resultArray, server, sp);
            }
          });
          for (_i = 0, _len = sanitizedParameters.length; _i < _len; _i++) {
            parameter = sanitizedParameters[_i];
            request.addParameter(parameter.name, Sapphire.parametersTypes[parameter.type], parameter.value);
          }
          request.on("row", function(columns) {
            var column, returnedValues, _j, _len1;
            returnedValues = {};
            for (_j = 0, _len1 = columns.length; _j < _len1; _j++) {
              column = columns[_j];
              returnedValues[column.metadata.colName] = column.value;
            }
            resultArray[rowIndex] = returnedValues;
            return rowIndex++;
          });
          return DBPM.execute(server, request);
        } else {
          return this.emit("databaseError", "Invalid parameter");
        }
      } else {
        return this.emit("serverNotOnList", server);
      }
    };

    Sapphire.prototype.execute = function() {
      if ((this.pq.server != null) && (this.pq.sp != null)) {
        return this.query(this.pq.server, this.pq.sp, this.pq.parameters);
      }
    };

    Sapphire.prototype.sanitize = function(parameters) {
      var newParameters, newValue, parameter, _i, _len;
      newParameters = parameters;
      for (_i = 0, _len = parameters.length; _i < _len; _i++) {
        parameter = parameters[_i];
        if (this.hasDataType(parameter.type)) {
          if (Util.typeOf(parameter.value) === "string") {
            newValue = Sanitization.trim(parameter.value);
            newValue = Sanitization.escapeQuotes(newValue);
            newValue = Sanitization.dropComments(newValue);
            newParameters[_i].value = newValue;
          }
        }
      }
      return newParameters;
    };

    Sapphire.prototype.isValid = function(parameters) {
      var currentType, currentValue, max, min, parameter, _i, _len;
      for (_i = 0, _len = parameters.length; _i < _len; _i++) {
        parameter = parameters[_i];
        currentValue = parameter.value;
        currentType = parameter.type;
        max = parameter.maxValue;
        min = parameter.minValue;
        if (currentValue == null) {
          this.emit("parameterNotDefined", parameter);
          return false;
        }
        if (!this.hasDataType(currentType)) {
          this.emit("invalidDataType", parameter);
          return false;
        }
        if (!Validation.isTypeOK(currentType, currentValue)) {
          this.emit("dataTypeMismatch", parameter);
          return false;
        }
        if (!Validation.isSizeOK(currentType, currentValue, min, max)) {
          this.emit("sizeOutOfBounds", parameter);
          return false;
        }
        if (parameter.onlyNumbers) {
          if (!Validation.isOnlyNumbers(currentValue)) {
            this.emit("notOnlyNumbers", parameter);
            return false;
          }
        }
        if (parameter.onlyLettersAndSymbols) {
          if (!Validation.isOnlyLettersAndSymbols(currentValue)) {
            this.emit("notOnlyLettersAndSymbols", parameter);
            return false;
          }
        }
        if (parameter.onlyLetters) {
          if (!Validation.isOnlyLetters(currentValue)) {
            this.emit("notOnlyLetters", parameter);
            return false;
          }
        }
      }
      return true;
    };

    return Sapphire;

  })(EventEmitter);

  module.exports = Sapphire;

}).call(this);
